#ifndef ToolTip_H
#define ToolTip_H

#include "Array.h"
#include "Zeichnung.h"
#include <functional>

namespace Framework
{
    class Schrift; // Schrift.h
    class Bild; // Bild.h
    class AlphaFeld; // AlphaFeld.h
    class Text; // Text.h
    class Rahmen; // Rahmen.h
    struct MausEreignis; // MausEreignis.h
    class Bildschirm; // Bildschirm.h

    class ToolTip : public ZeichnungHintergrund
    {
    private:
        RCArray< Zeichnung > *members;
        Punkt size;
        double animationSpeed;
        double warten;
        double wartenCount;
        double tval;
        bool mausIn;
        unsigned char alpha;
        bool sichtbar;
        bool zeichnen;
        bool mausIn2;
        Bildschirm *bildschirm;
        std::function< void( ToolTip * ) > onShow;
        std::function< void( ToolTip * ) > onHide;
        // Verarbeitet Maus Nachrichten
        //  me: Das Ereignis, was durch die Mauseingabe ausgelt wurde
        __declspec( dllexport ) void doMausEreignis( MausEreignis &me, bool userRet ) override;

    public:
        // Konstruktor
        __declspec( dllexport ) ToolTip( Bildschirm *zSceen );
        // Destruktor
        __declspec( dllexport ) ~ToolTip();
        // Fgt eine Zeichnung zum Tooltip hinzu
        //  m: die neue Zeichnung
        __declspec( dllexport ) void addMember( Zeichnung *m );
        // Entfernt eine Zeichnung vom Tooltip
        //  m: die Zeichnung
        __declspec( dllexport ) void removeMember( Zeichnung *zM );
        // Entfernt eine Zeichnung vom Tooltip
        //  i: der Index der Zeichnung
        __declspec( dllexport ) void removeMember( int i );
        // setzt eine Funktion, die aufgerufen wird, sobald der Tooltip angezeigt wird
        //  onShow: Die Funktion
        __declspec( dllexport ) void setShowEvent( std::function< void( ToolTip * ) > onShow );
        // setzt eine Funktion, die aufgerufen wird, sobald der Tooltip nicht mehr angezeigt wird
        //  onShow: Die Funktion
        __declspec( dllexport ) void setHideEvent( std::function< void( ToolTip * ) > onHide );
        // Setzt die anzahl an Seunden, de gewartet wird bis der Tipp erscheint
        //  warten: Die Anzahl an Sekunden
        __declspec( dllexport ) void setWarten( double warten );
        // Legt fest, wie schnell der Tipp erscheint
        //  speed: Anzahl der Pixel, die pro Sekunde eingeblendet werden. (Standart: 250)
        __declspec( dllexport ) void setAnimationSpeed( double speed );
        // Legt Fest, ob die Maus in dem Zeichnung ist, zu dem der Tip gehrt
        //  mausIn: 1, wenn die Maus in dem Zeichnung ist. 0 sonst
        __declspec( dllexport ) void setMausIn( bool mausIn );
        // Setzt den Counter zurck, der zhlt, wann der Tipp eingeblendet wird.
        __declspec( dllexport ) void wartenReset();
        // Legt fest, das das Zeichnung, zu dem der Tip gehrt gezeichnet wurde, so dass der Tip auch gezeichnet werden knnte
        __declspec( dllexport ) void setZeichnen();
        // Aktualisiert den Tip. Wird vom Framework aufgerufen
        //  tickVal: Die Zeit in sekunden, die seit dem lezten Aufruf dieser Funktion vergangen ist
        //  return: 1, wenn sich etwas verndert hat und das Bild neu gezeichnet werden muss. 0 sonst
        __declspec( dllexport ) bool tick( double tickVal ) override;
        // Zeichnet den Tip nach zRObj, falls er sichtbar ist und das zugehrige Zeichnung ebenfalls gezeichnet wurde
        //  zRObj: Das Bild, in welches gezeichnet werden soll
        __declspec( dllexport ) void render( Bild &zRObj ) override;
        // Gibt einen Zeiger auf den Bildschirm ohne erhhten Reference Counter zurck, so dem der Tooltip gehrt.
        __declspec( dllexport ) Bildschirm *zBildschirm() const;
        // Gibt ein bestimmten member zurck (ohne erhhten Reference Counter)
        //  i: der Index des Members
        __declspec( dllexport ) Zeichnung *zMember( int i ) const;
        // Gibt ein bestimmten member zurck
        //  i: der Index des Members
        __declspec( dllexport ) Zeichnung *getMember( int i ) const;
        // Gibt die Anzahl an Zeichnungen zurck, die zum Tooltip gehren
        __declspec( dllexport ) int getMemberAnzahl() const;
        // Erzeugt eine komplette kopie eines tooltips
        __declspec( dllexport ) Zeichnung *dublizieren() const override;
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) Zeichnung *release() override;
    };
}

#endif